/* ARCHIVO: biblioteca.c
*  DESCRIPCIN: Una biblioteca que mantiene sus obras en formato digital lo ha elegido a Ud. para automatizar su sistema de bsquedas.  La biblioteca almacena la informacin de sus obras en el archivo BIBLIOTECA.DAT.  Este archivo tiene el siguiente formato:
*
*		<Archivo>$<TituloObra>$<Edicion>$<PalabrasClave>$<Autor>$<Ao>
*
*		donde: 
*		-	<Archivo>: Indica la ruta del archivo que contiene determinada obra (Mx 255 caracteres).	
*		-	<TituloObra>: Indica el ttulo de la obra (Mx 100 caracteres).
*		-	<Edicion>: Indica el nmero de edicin de la obra.
*		-	<PalabrasClave>: Es una lista de mximo 5 palabras clave separadas por comas (Cada palabra tiene como mximo 20 caracteres).
*		-	<Autor>: Indica el nombre del autor de la obra (Mx 100 caracteres).
*		-	<Ao>: Indica el ao de publicacin de la obra.
*		La biblioteca ofrece el servicio de bsqueda e impresin de sus obras, por lo cual se le solicita a usted realizar un programa que:
*
*		a)	Lea los datos del archivo BIBLIOTECA.DAT con el formato indicado anteriormente y los cargue a memoria.
*		b)	Permita realizar bsqueda de obras: 
*		1.	Por ttulo 
*		2.	Por autor.
*		3.	Por palabras claves
* 		El resultado de la bsqueda debe ser presentado de la siguiente manera:
*
*		TITULO                                EDICION		AUTOR 			AO
*		-----------------------------------------------------------------------------------------------------------------------------
*
*		c)	Imprima el contenido de determinada obra.
*
*		NOTA:
*		Ud. dispone de la librera printer.lib que contiene la funcin bool PrintFile(FILE *pFile), la cual enva a la impresora predeterminada el archivo referenciado por pFile.  La interfaz de la librera est definida en printer.h
*
*
*
*  AUTOR: Luis Galrraga Del Prado
*  FECHA DE CREACIN: 26-12-2004
*/

#include <stdio.h>
#include <simpio.h>
#include <conio.h>

//Entidades reconocidas en el problema
typedef struct{
	char ruta[256];
	char titulo[101];
	int edicion;
	char palabrasclave[106];
	char autor[101];
	int anio;
}Obra;


/*Funciones y Procedimientos*/
void CargarDatos(Obra *D,int longitud, FILE *infile);
void ConsultarXTitulo(Obra *D, int longitud, string clave);
void ConsultarXAutor(Obra *D, int longitud, string clave);
void ConsultarXPalabrasClaves(Obra *D, int longitud, string clave);
int ContarObras(FILE *archivo);

void main(){
	Obra *Datos;
	int op,longitud;
	FILE *respaldo;
	string clave;
	respaldo=fopen("biblioteca.dat","r");
	if(!respaldo){ //Es equivalente a escribir if(respaldo==NULL)
		printf("No se pudo abrir el archivo biblioteca.dat");
		getch();
		exit(2);
	}

	do{
		printf("Ingrese 1 para consultar obra por t%ctulo\n",161);
		printf("Ingrese 2 para consultar obra por autor\n");
		printf("Ingrese 3 para consultar obra por palabras claves\n");
		printf("Ingrese 4 para salir\n");
		printf("Opci%cn :",162);
		op=GetInteger();
	}while(op<1 || op>4);
	if(op==4){
		printf("\nUsted ha decidido salir");
		exit(4);
	}
	longitud=ContarObras(respaldo); //Cuento el nmero de obras
	rewind(respaldo);
	Datos=malloc(longitud*sizeof(Obra)); //Compro memoria para los datos
	CargarDatos(Datos,longitud,respaldo); //Cargo los datos en el arreglo
	printf("\nIngrese la clave: ");
	clave=GetLine();
	switch(op){
		case 1:
			ConsultarXTitulo(Datos,longitud,clave);
			break;
		case 2:
			ConsultarXAutor(Datos,longitud,clave);
			break;
		case 3:
			ConsultarXPalabrasClaves(Datos,longitud, clave);
			break;
	}

	free(Datos);
	fclose(respaldo);
	
}

/* FUNCIN: CargarDatos(Obra *D, FILE *infile);
*  DESCRIPCIN: Realiza la descarga de las obras almacenadas en el archivo
*				referenciado por infile en el arreglo dinmico de Obras D 
*/

void CargarDatos(Obra *D,int longitud, FILE *infile){
	int ns,c=0;
	char salto;
	while(TRUE){
		ns=fscanf(infile,"%255[^$]$%100[^$]$%d$%105[^$]$%100[^$]$%d%c",D[c].ruta,D[c].titulo,&(D[c].edicion),D[c].palabrasclave,D[c].autor,&(D[c].anio),&salto);
		if(salto==EOF) break;
		if(ns!=6 && salto!='\n')
			printf("Error de formato");
		c++;
		if(c==longitud) break;
	}
}

/* FUNCIN: ContarObras(FILE *infile)
*  DESCRIPCIN: Retorna el nmero de lneas del archivo apuntado por infile.
*/
int ContarObras(FILE *archivo){
	char linea[1000];
	int c=0;

	while(!feof(archivo)){
		fgets(linea,1000,archivo);
		c++;
	}
	return(c);
}

/* PROCEDIMIENTO: ConsultarXTitulo(Obra *D, int longitud, char *clave)
*  DESCRIPCIN: Genera como salida un archivo con el resultado de la consulta
*				de videos por ttulo
*/

void ConsultarXTitulo(Obra *D, int longitud, string clave){
	FILE *outfile;
	int k;

	outfile=fopen("consultaxtitulo.txt","w");
	fprintf(outfile,"TITULO\tEDICIN\t\tAUTOR\tAO\n");
	fprintf(outfile,"------------------------------------------\n");

	for(k=0;k<longitud;k++){
		if(StringEqual(D[k].titulo,clave))
			fprintf(outfile,"%s\t%d\t\t%s\t%d\n",D[k].titulo,D[k].edicion, D[k].autor, D[k].anio);	
	}
	fclose(outfile);

}

/* PROCEDIMIENTO: ConsultarXAutor(Obra *D, int longitud, char *clave)
*  DESCRIPCIN: Genera como salida un archivo con el resultado de la consulta
*				de videos por autor
*/

void ConsultarXAutor(Obra *D, int longitud, string clave){
	FILE *outfile;
	int k;

	outfile=fopen("consultaxautor.txt","w");
	fprintf(outfile,"TITULO\tEDICIN\t\tAUTOR\tAO\n");
	fprintf(outfile,"------------------------------------------\n");

	for(k=0;k<longitud;k++){
		if(StringEqual(D[k].autor,clave))
			fprintf(outfile,"%s\t%d\t\t%s\t%d\n",D[k].titulo,D[k].edicion, D[k].autor, D[k].anio);	
	}
	fclose(outfile);

}

/* PROCEDIMIENTO: ConsultarXPalabrasClaves(Obra *D, int longitud, char *clave)
*  DESCRIPCIN: Genera como salida un archivo con el resultado de la consulta
*				de videos por palabras claves
*/

void ConsultarXPalabrasClaves(Obra *D, int longitud, string clave){
	FILE *outfile;
	int k;

	outfile=fopen("consultaxpalabrasclaves.txt","w");
	fprintf(outfile,"TITULO\tEDICIN\t\tAUTOR\tAO\n");
	fprintf(outfile,"------------------------------------------\n");

	for(k=0;k<longitud;k++){
		if(FindString(clave, D[k].palabrasclave,0)>=0)
			fprintf(outfile,"%s\t%d\t\t%s\t%d\n",D[k].titulo,D[k].edicion, D[k].autor, D[k].anio);	
	}
	fclose(outfile);
}