/* ARCHIVO: palabra_digital.c
*  DESCRIPCIN:
*  	      FUNDASOFT ha recibido el requerimiento de escribir un programa que dada una 
*	      cadena de caracteres permita generar un archivo que contenga esa  cadena en 
*	      un formato digital. Por ejemplo;  si se ingresa la palabra  sapo, se deber 
*	      originar un archivo cuyo contenido luce como se indica en sapo.txt.
*	      Existe un archivo llamado letras.dat que contiene todas las letras del al--
*	      fabeto y tiene el siguiente formato:
*	      
*	      Letra,cadenas de 1s y 0s
*	      
*	      Un ejemplo de lnea del archivo sera:
*	      
*	      A,1111110001111111000110001
*  AUTOR: Luis Galrraga Del Prado
*  FECHA DE CREACIN: 09-octubre-2005
*/

#include <stdio.h>
#include <string.h>

#define MAX_LETRAS 9

/* Estructura definida en el problema (parte del ejercicio)
*/
typedef struct{
	int Matriz[5][5];
	char letra;
}LetraDigital;

//Funciones que se piden implementar

/* PROCEDIMIENTO: CargarLetras
*  DESCRIPCIN: Procedimiento que lee el archivo asociado con el puntero fLetras y llena 
*		el arreglo aLetras con la representacin digital de las letras del abece
*		dario.
*/
void CargarLetras(FILE *fLetras, LetraDigital aLetras[], int MAX);

/* FUNCIN: DigitalizarPalabra
*  DESCRIPCIN: Funcin que dada una palabra retorna el arreglo de letras digitales que la
*		formen.
*/
LetraDigital* DigitalizarPalabra(char *palabra, LetraDigital aLetras[], int MAX);

/* FUNCIN: AlmacenarPalabraDigital
*  DESCRIPCIN: Procedimiento que dada una palabra, genera un archivo llamado <palabra.dat>
*		que contiene la representacin digital de la palabra.
*/
void AlmacenarPalabraDigital(char *palabra, LetraDigital aLetras[], int MAX);

//Funciones auxiliares
void LetraDigital_Imprimirf(FILE *salida, LetraDigital PD, int n_espacios);
void LetraDigital_Copiar(LetraDigital *destino, LetraDigital fuente);
int BuscarLetraDigital(char letra, LetraDigital aLetras[], int MAX);
void LetraDigital_LlenarMatriz(LetraDigital *LD, char *cadena);

//Nota: No piden implementarlo, lo hago solo para probar las funciones
void main(){
	char palabra[100];
	LetraDigital baseDatos[MAX_LETRAS];
	FILE *fLetras;

	printf("Ingrese la palabra que desea digitalizar: ");
	scanf("%s",palabra);
	
	fLetras = fopen("letras.dat","r");
	if(!fLetras){
		printf("No se pudo abrir el archivo letras.dat");
		return;
	}
	CargarLetras(fLetras,baseDatos,MAX_LETRAS);

	AlmacenarPalabraDigital(palabra,baseDatos,MAX_LETRAS);
	
	fclose(fLetras);
	getch();
}

/*				RESOLUCIN
----------------------------------------------------------------------------
----------------------------------------------------------------------------*/

void CargarLetras(FILE *fLetras, LetraDigital aLetras[], int MAX){
	int i = 0;
	char binario[26];

	while(!feof(fLetras) && i<MAX){
		fscanf(fLetras,"%c,%[^\n]\n", &(aLetras[i].letra),binario);
		//Leo los unos y ceros como cadena y los coloco en la matriz
		LetraDigital_LlenarMatriz(&(aLetras[i]),binario);
		i++;
	}

}

/* FUNCIN: LetraDigital_LlenarMatriz
*  DESCRIPCIN: Recibe la direccin de memoria de una palabra digital y una cadena de
*		unos y ceros y llena la matriz de enteros de la con la informacin de
*		la cadena. 
*/
void LetraDigital_LlenarMatriz(LetraDigital *LD, char *cadena){
	int i,j;

	for(i=0;i<5;i++){
		for(j=0;j<5;j++){
			if(cadena[i*5+j] == '1')
				LD->Matriz[i][j] = 1;
			else
				LD->Matriz[i][j] = 0;
				
		}
	}
}

LetraDigital* DigitalizarPalabra(char *palabra, LetraDigital aLetras[], int MAX){
	LetraDigital *digitalizada;
	int i, size, id;
	
	size = strlen(palabra);
	digitalizada = malloc(strlen(palabra)*sizeof(LetraDigital));
	
	for(i=0;i<size;i++){
		//Obtengo el ndice del arreglo
		id = BuscarLetraDigital(toupper(palabra[i]),aLetras,MAX);
		if(id>-1)
			//Copio la letra digital
			LetraDigital_Copiar(&(digitalizada[i]),aLetras[id]);
	}
	return digitalizada;
}


/* FUNCIN: BuscarLetraDigital
*  DESCRIPCIN: Recibe un caracter y un arreglo de Letras digitales y retorna el ndice
*		en el arreglo de la letra digital que representa al caracter. Recibe --
*		adems el tamao del arreglo. De no encontrar la clave la funcin retorna
*		-1.
*/
int BuscarLetraDigital(char letra, LetraDigital aLetras[], int MAX){
	int i;
	for(i=0;i<MAX;i++){
		if(letra == aLetras[i].letra)
			return i;
	}
	return -1;
}

/* FUNCIN: LetraDigital_Copiar
*  DESCRIPCIN: Copia el contenido de la Letra Digital fuente en la destino
*/
void LetraDigital_Copiar(LetraDigital *destino, LetraDigital fuente){
	int i,j;
	destino->letra = fuente.letra;
	for(i=0;i<5;i++){
		for(j=0;j<5;j++)
			destino->Matriz[i][j]=fuente.Matriz[i][j];
	}
}

void AlmacenarPalabraDigital(char *palabra, LetraDigital aLetras[], int MAX){
	char *filename;
	FILE *outfile;
	int size,i;
	LetraDigital *palabradigital;
	
	//Generacin del nombre del archivo
	size = strlen(palabra);
	filename = malloc((size+5)*sizeof(char));
	strcpy(filename,palabra);
	strcat(filename,".txt");

	outfile = fopen(filename,"w");
	palabradigital = DigitalizarPalabra(palabra,aLetras,MAX);
	
	//Impresin de la palabra digital en el archivo
	for(i=0;i<size;i++)
		LetraDigital_Imprimirf(outfile,palabradigital[i],i*6);
	
	fclose(outfile);
	free(palabradigital);
}

/* PROCEDIMIENTO: LetraDigital_Imprimirf
*  DESCRIPCIN: Recibe una letra digital e imprime su representacin en el archivo referenciado
*		por 'salida'. El otro parmetro sirve para determinar el nmero de espacios que 
*		deben imprimirse antes de la letra.
*/
void LetraDigital_Imprimirf(FILE *salida, LetraDigital PD, int n_espacios){
	int i,j,k;
	for(i=0;i<5;i++){
		//Impresin de los espacios
		for(k=0;k<n_espacios;k++)
			fprintf(salida," ");
		
		for(j=0;j<5;j++){
			if(PD.Matriz[i][j]==1)
				fputc('*',salida);
			else
				fputc(' ',salida);
		}

		fprintf(salida,"\n");
	}
}	
	


